// src/App.js
import React, { useState, useEffect } from 'react';
import { BrowserRouter as Router, Route, Routes, Navigate, useNavigate } from 'react-router-dom';
import { useParams } from 'react-router-dom';
import { jwtDecode } from 'jwt-decode';
import Login from './components/Login';
import Boxes from './components/Boxes';
import Items from './components/Items';
import Navbar from './components/Navbar';
import Admin from './components/Admin';
import { createContext } from 'react';
import './styles.css'

export const AppContext = createContext();
export const PRIMARY_COLOR = '#333';
export const SECONDARY_COLOR = '#ffffff';
export const BACKGROUND_COLOR = '#dddddd';

// New component for token expiration check
function TokenExpirationCheck({ setToken }) {
  const navigate = useNavigate();

  useEffect(() => {
    const checkTokenExpiration = () => {
      const token = localStorage.getItem('token');
      if (!token) return;
      
      try {
        const decodedToken = jwtDecode(token);
        if (Date.now() >= decodedToken.exp * 1000) {
          setToken(null);
          localStorage.removeItem('token');
          navigate('/api/v1/login');
        }
      } catch (error) {
        localStorage.removeItem('token');
        navigate('/api/v1/login');
      }
    };

    window.addEventListener('focus', checkTokenExpiration);
    const interval = setInterval(checkTokenExpiration, 60000);
    
    return () => {
      window.removeEventListener('focus', checkTokenExpiration);
      clearInterval(interval);
    };
  }, [navigate, setToken]);

  return null;
}

function App() {
  const [token, setToken] = useState(localStorage.getItem('token'));

  useEffect(() => {
    if (token) {
      localStorage.setItem('token', token);
    }
  }, [token]);

  return (
    <AppContext.Provider value={{ token, setToken }}>
      <Router>
        <TokenExpirationCheck setToken={setToken} />
        <Navbar />
        <AppRoutes token={token} setToken={setToken} />
      </Router>
    </AppContext.Provider>
  );
}

function AppRoutes({ token, setToken }) {
  const { id } = useParams();

  return (
    <>
      <Routes>
        <Route path="/api/v1/login" element={<Login setToken={setToken} />} />
        <Route 
          path="/api/v1/boxes" 
          element={token ? <Boxes token={token} /> : <Navigate to="/api/v1/login" replace />} 
        />
        <Route 
          path="/api/v1/items" 
          element={token ? <Items token={token} /> : <Navigate to="/api/v1/login" replace />} 
        />
        <Route 
          path="/api/v1/boxes/:id/items" 
          element={token ? <Items box_id={id} token={token} /> : <Navigate to="/api/v1/login" replace />} 
        />
        <Route 
          path="/api/v1/admin" 
          element={token ? <Admin token={token}/> : <Navigate to="/api/v1/login" replace />} 
        />
        <Route path="*" element={<Navigate to={token ? "/api/v1/boxes" : "/api/v1/login"} replace />} />
      </Routes>
    </>
  );
}

export default App;